@@ -1,5 +1,5 @@
-from pydantic import BaseSettings, Field
-
+from pydantic import BaseSettings
+from typing import List
 
 TELEMETRY_WHITELISTED_SETTINGS = ["chroma_db_impl", "chroma_api_impl", "chroma_server_ssl_enabled"]
 
@@ -19,6 +19,7 @@ class Settings(BaseSettings):
     chroma_server_http_port: str = None
     chroma_server_ssl_enabled: bool = False
     chroma_server_grpc_port: str = None
+    chroma_server_cors_allow_origins: List[str] = []  # eg ["http://localhost:3000"]
 
     anonymized_telemetry: bool = True
 
