@@ -1,13 +1,10 @@
 from chromadb.api.types import Documents, Embeddings, IDs, Metadatas, Where, WhereDocument
 from chromadb.db import DB
-from chromadb.db.index.hnswlib import Hnswlib
+from chromadb.db.index.hnswlib import Hnswlib, delete_all_indexes
 from chromadb.errors import (
     NoDatapointsException,
-    InvalidDimensionException,
-    NotEnoughElementsException,
 )
 import uuid
-import time
 import numpy.typing as npt
 import json
 from typing import Dict, Optional, Sequence, List, Tuple, cast
@@ -51,7 +48,6 @@ class Clickhouse(DB):
     #
     def __init__(self, settings):
         self._conn = None
-        self._idx = Hnswlib(settings)
         self._settings = settings
 
     def _init_conn(self):
@@ -81,6 +77,25 @@ class Clickhouse(DB):
         ) ENGINE = MergeTree() ORDER BY collection_uuid"""
         )
 
+    index_cache = {}
+
+    def _index(self, collection_id):
+        """Retrieve an HNSW index instance for the given collection"""
+
+        if collection_id not in self.index_cache:
+            coll = self.get_collection_by_id(collection_id)
+            collection_metadata = coll[2]
+            index = Hnswlib(collection_id, self._settings, collection_metadata)
+            self.index_cache[collection_id] = index
+
+        return self.index_cache[collection_id]
+
+    def _delete_index(self, collection_id):
+        """Delete an index from the cache"""
+        index = self._index(collection_id)
+        index.delete()
+        del self.index_cache[collection_id]
+
     #
     #  UTILITY METHODS
     #
@@ -156,8 +171,21 @@ class Clickhouse(DB):
         # json.loads the metadata
         return [[x[0], x[1], json.loads(x[2])] for x in res]
 
+    def get_collection_by_id(self, collection_uuid: str):
+        res = (
+            self._get_conn()
+            .query(
+                f"""
+         SELECT * FROM collections WHERE uuid = '{collection_uuid}'
+         """
+            )
+            .result_rows
+        )
+        # json.loads the metadata
+        return [[x[0], x[1], json.loads(x[2])] for x in res][0]
+
     def list_collections(self) -> Sequence:
-        res = self._get_conn().query(f"""SELECT * FROM collections""").result_rows
+        res = self._get_conn().query("SELECT * FROM collections").result_rows
         return [[x[0], x[1], json.loads(x[2])] for x in res]
 
     def update_collection(
@@ -189,14 +217,14 @@ class Clickhouse(DB):
         """
         )
 
+        self._delete_index(collection_uuid)
+
         self._get_conn().command(
             f"""
          DELETE FROM collections WHERE name = '{name}'
          """
         )
 
-        self._idx.delete_index(collection_uuid)
-
     #
     #  ITEM METHODS
     #
@@ -272,8 +300,8 @@ class Clickhouse(DB):
         # Update the index
         if embeddings is not None:
             update_uuids = [x[1] for x in existing_items]
-            self._idx.delete_from_index(collection_uuid, update_uuids)
-            self._idx.add_incremental(collection_uuid, update_uuids, embeddings)
+            index = self._index(collection_uuid)
+            index.add(update_uuids, embeddings, update=True)
 
     def _get(self, where={}, columns: Optional[List] = None):
         select_columns = db_schema_to_keys() if columns is None else columns
@@ -367,14 +395,12 @@ class Clickhouse(DB):
         where_document: WhereDocument = {},
         columns: Optional[List[str]] = None,
     ) -> Sequence:
-        if collection_name == None and collection_uuid == None:
+        if collection_name is None and collection_uuid is None:
             raise TypeError("Arguments collection_name and collection_uuid cannot both be None")
 
         if collection_name is not None:
             collection_uuid = self.get_collection_uuid_from_name(collection_name)
 
-        s3 = time.time()
-
         where_str = self._create_where_clause(
             # collection_uuid must be defined at this point, cast it for typechecker
             cast(str, collection_uuid),
@@ -386,7 +412,7 @@ class Clickhouse(DB):
         if sort is not None:
             where_str += f" ORDER BY {sort}"
         else:
-            where_str += f" ORDER BY collection_uuid"  # stable ordering
+            where_str += " ORDER BY collection_uuid"  # stable ordering
 
         if limit is not None or isinstance(limit, int):
             where_str += f" LIMIT {limit}"
@@ -426,7 +452,6 @@ class Clickhouse(DB):
         ids: Optional[IDs] = None,
         where_document: WhereDocument = {},
     ) -> List:
-        s3 = time.time()
         where_str = self._create_where_clause(
             # collection_uuid must be defined at this point, cast it for typechecker
             cast(str, collection_uuid),
@@ -437,7 +462,8 @@ class Clickhouse(DB):
 
         deleted_uuids = self._delete(where_str)
 
-        self._idx.delete_from_index(collection_uuid, deleted_uuids)
+        index = self._index(collection_uuid)
+        index.delete_from_index(deleted_uuids)
 
         return deleted_uuids
 
@@ -470,27 +496,12 @@ class Clickhouse(DB):
     ) -> Tuple[List[List[uuid.UUID]], npt.NDArray]:
 
         # Either the collection name or the collection uuid must be provided
-        if collection_name == None and collection_uuid == None:
+        if collection_name is None and collection_uuid is None:
             raise TypeError("Arguments collection_name and collection_uuid cannot both be None")
 
         if collection_name is not None:
             collection_uuid = self.get_collection_uuid_from_name(collection_name)
 
-        self._idx.load_if_not_loaded(collection_uuid)
-
-        idx_metadata = self._idx.get_metadata()
-        # Check query embeddings dimensionality
-        if idx_metadata["dimensionality"] != len(embeddings[0]):
-            raise InvalidDimensionException(
-                f"Query embeddings dimensionality {len(embeddings[0])} does not match index dimensionality {idx_metadata['dimensionality']}"
-            )
-
-        # Check number of requested results
-        if n_results > idx_metadata["elements"]:
-            raise NotEnoughElementsException(
-                f"Number of requested results {n_results} cannot be greater than number of elements in index {idx_metadata['elements']}"
-            )
-
         if len(where) != 0 or len(where_document) != 0:
             results = self.get(
                 collection_uuid=collection_uuid, where=where, where_document=where_document
@@ -504,9 +515,9 @@ class Clickhouse(DB):
                 )
         else:
             ids = None
-        uuids, distances = self._idx.get_nearest_neighbors(
-            collection_uuid, embeddings, n_results, ids
-        )
+
+        index = self._index(collection_uuid)
+        uuids, distances = index.get_nearest_neighbors(embeddings, n_results, ids)
 
         return uuids, distances
 
@@ -523,13 +534,16 @@ class Clickhouse(DB):
         uuids = [x[1] for x in get]
         embeddings = [x[2] for x in get]
 
-        self._idx.run(collection_uuid, uuids, embeddings)
+        index = self._index(collection_uuid)
+        index.add(uuids, embeddings)
 
     def add_incremental(self, collection_uuid, uuids, embeddings):
-        self._idx.add_incremental(collection_uuid, uuids, embeddings)
+        index = self._index(collection_uuid)
+        index.add(uuids, embeddings)
 
-    def has_index(self, collection_uuid: str):
-        return self._idx.has_index(collection_uuid)
+    def reset_indexes(self):
+        delete_all_indexes(self._settings)
+        self.index_cache = {}
 
     def reset(self):
         conn = self._get_conn()
@@ -538,8 +552,7 @@ class Clickhouse(DB):
         self._create_table_collections(conn)
         self._create_table_embeddings(conn)
 
-        self._idx.reset()
-        self._idx = Hnswlib(self._settings)
+        self.reset_indexes()
 
     def raw_sql(self, sql):
         return self._get_conn().query(sql).result_rows
