@@ -1,4 +1,3 @@
-import pytest
 import unittest
 import os
 from unittest.mock import patch
@@ -10,19 +9,19 @@ import chromadb.config
 class GetDBTest(unittest.TestCase):
     @patch("chromadb.db.duckdb.DuckDB", autospec=True)
     def test_default_db(self, mock):
-        db = chromadb.get_db(chromadb.config.Settings(persist_directory="./foo"))
+        chromadb.get_db(chromadb.config.Settings(persist_directory="./foo"))
         assert mock.called
 
     @patch("chromadb.db.duckdb.PersistentDuckDB", autospec=True)
     def test_persistent_duckdb(self, mock):
-        db = chromadb.get_db(
+        chromadb.get_db(
             chromadb.config.Settings(chroma_db_impl="duckdb+parquet", persist_directory="./foo")
         )
         assert mock.called
 
     @patch("chromadb.db.clickhouse.Clickhouse", autospec=True)
     def test_clickhouse(self, mock):
-        db = chromadb.get_db(
+        chromadb.get_db(
             chromadb.config.Settings(
                 chroma_db_impl="clickhouse",
                 persist_directory="./foo",
@@ -38,14 +37,14 @@ class GetAPITest(unittest.TestCase):
     @patch("chromadb.api.local.LocalAPI", autospec=True)
     @patch.dict(os.environ, {}, clear=True)
     def test_local(self, mock_api, mock_db):
-        api = chromadb.Client(chromadb.config.Settings(persist_directory="./foo"))
+        chromadb.Client(chromadb.config.Settings(persist_directory="./foo"))
         assert mock_api.called
         assert mock_db.called
 
     @patch("chromadb.api.fastapi.FastAPI", autospec=True)
     @patch.dict(os.environ, {}, clear=True)
     def test_fastapi(self, mock):
-        api = chromadb.Client(
+        chromadb.Client(
             chromadb.config.Settings(
                 chroma_api_impl="rest",
                 persist_directory="./foo",
