@@ -4,7 +4,7 @@ from chromadb.api.types import Documents, EmbeddingFunction, Embeddings
 class SentenceTransformerEmbeddingFunction(EmbeddingFunction):
     # If you have a beefier machine, try "gtr-t5-large".
     # for a full list of options: https://huggingface.co/sentence-transformers, https://www.sbert.net/docs/pretrained_models.html
-    def __init__(self, model_name: str = "all-MiniLM-L6-v2"): 
+    def __init__(self, model_name: str = "all-MiniLM-L6-v2"):
         try:
             from sentence_transformers import SentenceTransformer
         except ImportError:
@@ -61,27 +61,32 @@ class CohereEmbeddingFunction(EmbeddingFunction):
             embeddings for embeddings in self._client.embed(texts=texts, model=self._model_name)
         ]
 
+
 class HuggingFaceEmbeddingFunction(EmbeddingFunction):
     def __init__(self, api_key: str, model_name: str = "sentence-transformers/all-MiniLM-L6-v2"):
         try:
-            import requests 
+            import requests
         except ImportError:
             raise ValueError(
                 "The requests python package is not installed. Please install it with `pip install requests`"
             )
-        self._api_url = f"https://api-inference.huggingface.co/pipeline/feature-extraction/{model_name}" 
+        self._api_url = (
+            f"https://api-inference.huggingface.co/pipeline/feature-extraction/{model_name}"
+        )
         self._session = requests.Session()
         self._session.headers.update({"Authorization": f"Bearer {api_key}"})
 
     def __call__(self, texts: Documents) -> Embeddings:
         # Call HuggingFace Embedding API for each document
-        return self._session.post(self._api_url, json={"inputs": texts, "options":{"wait_for_model":True}}).json()
+        return self._session.post(
+            self._api_url, json={"inputs": texts, "options": {"wait_for_model": True}}
+        ).json()
 
 
 class InstructorEmbeddingFunction(EmbeddingFunction):
     # If you have a GPU with at least 6GB try model_name = "hkunlp/instructor-xl" and device = "cuda"
     # for a full list of options: https://github.com/HKUNLP/instructor-embedding#model-list
-    def __init__(self, model_name: str = "hkunlp/instructor-base", device= "cpu"): 
+    def __init__(self, model_name: str = "hkunlp/instructor-base", device="cpu"):
         try:
             from InstructorEmbedding import INSTRUCTOR
         except ImportError:
